@@ -44,6 +44,10 @@
 
 extern GstTool *tool;
 
+static gboolean is_user_in_group (OobsUser  *user,
+				  OobsGroup *group);
+
+
 static gboolean
 check_user_delete (OobsUser *user)
 {
@@ -153,7 +157,7 @@ set_entry_text (GtkWidget *entry, const gchar *text)
 }
 
 static void
-set_main_group (OobsUser *user)
+select_main_group (OobsUser *user)
 {
 	GtkWidget *combo;
 	GtkTreeModel *model;
@@ -195,43 +199,67 @@ set_main_group (OobsUser *user)
 		gtk_combo_box_set_active (GTK_COMBO_BOX (combo), -1);
 }
 
-static OobsGroup*
-get_main_group (const gchar *name)
+static gboolean
+set_main_group (OobsUser *user)
 {
 	GtkWidget *combo;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	OobsGroup *group;
+	OobsGroupsConfig *config;
+	OobsList *groups_list;
+	OobsListIter list_iter;
+	const gchar *name;
+	gboolean is_group_new = FALSE;
 
+
+	name = oobs_user_get_login_name (user);
 	combo = gst_dialog_get_widget (tool->main_dialog, "user_settings_group");
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 
 	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter)) {
-		/* create new group for the user */
-		OobsGroupsConfig *config;
-		OobsList *groups_list;
-		OobsListIter list_iter;
+		group =	group_settings_get_group_from_name (name);
+		/* Most standard case when creating user:
+		 * no group using the login already exists, so create it */
+		if (group == NULL) {
+			is_group_new = TRUE;
+			group = oobs_group_new (name);
+			oobs_group_set_gid (group, group_settings_find_new_gid ());
 
-		group = oobs_group_new (name);
-		oobs_group_set_gid (group, group_settings_find_new_gid ());
-
-		/* FIXME: this should be in a generic function */
-		config = OOBS_GROUPS_CONFIG (GST_USERS_TOOL (tool)->groups_config);
-		groups_list = oobs_groups_config_get_groups (config);
-		oobs_list_append (groups_list, &list_iter);
-		oobs_list_set (groups_list, &list_iter, group);
-
-		groups_table_add_group (group, &list_iter);
-		gst_tool_commit (tool, OOBS_OBJECT (config));
-
-		return group;
+			/* FIXME: this should be in a generic function */
+			config = OOBS_GROUPS_CONFIG (GST_USERS_TOOL (tool)->groups_config);
+			groups_list = oobs_groups_config_get_groups (config);
+			oobs_list_append (groups_list, &list_iter);
+			oobs_list_set (groups_list, &list_iter, group);
+		}
+		/* Else group exists, use it */
+	}
+	else { /* Group has been chosen in the list, use it */
+		gtk_tree_model_get (model, &iter,
+				    COL_GROUP_OBJECT, &group,
+				    -1);
 	}
 
-	gtk_tree_model_get (model, &iter,
-			    COL_GROUP_OBJECT, &group,
-			    -1);
+	if (!is_user_in_group (user, group))
+		oobs_group_add_user (group, user);
 
-	return group;
+	/* Try to commit before doing anything, to avoid displaying wrong data */
+	if (is_group_new) {
+		if (gst_tool_commit (tool, GST_USERS_TOOL (tool)->groups_config) == OOBS_RESULT_OK)
+			groups_table_add_group (group, &list_iter);
+		else {
+			/* Something bad is happening, don't try to create the user.
+			 * gst_tool_commit () must already have displayed an error. */
+			g_object_unref (group);
+			return FALSE;
+		}
+	}
+
+	/* This will be committed with the user himself */
+	oobs_user_set_main_group (user, group);
+
+	g_object_unref (group);
+	return TRUE;
 }
 
 /* Retrieve the NO_PASSWD_LOGIN_GROUP.
@@ -373,12 +401,17 @@ user_settings_dialog_new (OobsUser *user)
 {
 	OobsUsersConfig *config;
 	OobsGroup *no_passwd_login_group;
-	GtkWidget *dialog, *widget;
+	GtkWidget *dialog, *widget, *notice;
 	const gchar *login = NULL;
 	gchar *title;
 	gint uid;
 
 	dialog = gst_dialog_get_widget (tool->main_dialog, "user_settings_dialog");
+	notice = gst_dialog_get_widget (tool->main_dialog, "user_settings_uid_disabled");
+
+	/* Set this before setting the UID so that it's not rejected */
+	widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_uid");
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (widget), 0, G_MAXINT32);
 
 	if (!user) {
 		g_object_set_data (G_OBJECT (dialog), "user", NULL);
@@ -393,6 +426,9 @@ user_settings_dialog_new (OobsUser *user)
 		uid = find_new_uid (GST_USERS_TOOL (tool)->minimum_uid,
 				    GST_USERS_TOOL (tool)->maximum_uid);
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), uid);
+		gst_dialog_try_set_sensitive (tool->main_dialog, widget, TRUE);
+		gtk_widget_hide (notice);
+
 		setup_profiles_visibility (tool, TRUE);
 	} else {
 		g_object_set_data_full (G_OBJECT (dialog), "user",
@@ -409,11 +445,27 @@ user_settings_dialog_new (OobsUser *user)
 
 		widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_uid");
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), oobs_user_get_uid (user));
+		/* Show a notice if the user is logged in,
+		 * except if we don't have the required permissions to edit UID anyway */
+		if (is_user_root (user)) {
+			gst_dialog_try_set_sensitive (tool->main_dialog, widget, FALSE);
+			gtk_widget_hide (notice);
+		}
+		else if (oobs_user_get_active (user) &&
+			 gst_dialog_is_authenticated (tool->main_dialog)) {
+			gst_dialog_try_set_sensitive (tool->main_dialog, widget, FALSE);
+			gtk_widget_show (notice);
+		}
+		else {
+			gst_dialog_try_set_sensitive (tool->main_dialog, widget, TRUE);
+			gtk_widget_hide (notice);
+		}
+
 		setup_profiles_visibility (tool, FALSE);
 	}
 
 	privileges_table_set_from_user (user);
-	set_main_group (user);
+	select_main_group (user);
 
 	widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_name");
 	set_entry_text (widget, login);
@@ -531,11 +583,18 @@ check_login (gchar **primary_text, gchar **secondary_text, gpointer data)
 {
 	OobsUser *user;
 	GtkWidget *widget;
+	GtkWidget *combo;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
 	const gchar *login;
+	gchar *group;
+
+	user = g_object_get_data (G_OBJECT (data), "user");
 
 	widget = gst_dialog_get_widget (tool->main_dialog, "user_settings_name");
 	login = gtk_entry_get_text (GTK_ENTRY (widget));
-	user = g_object_get_data (G_OBJECT (data), "user");
+
+	combo = gst_dialog_get_widget (tool->main_dialog, "user_settings_group");
 
 	if (strlen (login) < 1) {
 		*primary_text = g_strdup (_("User name is empty"));
@@ -548,7 +607,10 @@ check_login (gchar **primary_text, gchar **secondary_text, gpointer data)
 	} else if (!user && login_exists (login)) {
 		*primary_text = g_strdup_printf (_("User name \"%s\" already exists"), login);
 		*secondary_text = g_strdup (_("Please choose a different user name."));
-	} else if (!user && group_settings_group_exists (login)) {
+	/* Check that the new user won't be added to an already existing group
+	 * without that group being explicitly chosen in the list (temporary security check) */
+	} else if (!user && gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) == -1
+	                 && group_settings_group_exists (login)) {
 		*primary_text = g_strdup_printf (_("Group \"%s\" already exists"), login);
 		*secondary_text = g_strdup (_("Please choose a different user name."));
 	}
@@ -775,10 +837,10 @@ user_settings_dialog_get_data (GtkWidget *dialog)
 	if (no_passwd_login_group)
 		g_object_unref (no_passwd_login_group);
 
-	/* set main group */
-	group = get_main_group (oobs_user_get_login_name (user));
-	oobs_user_set_main_group (user, group);
-	g_object_unref (group);
+	/* If FALSE, group could not be found or created, which means
+	 * we won't be able to create the new user anyway, so stop here */
+	if (!set_main_group (user))
+		return NULL;
 
 	privileges_table_save (user);
 
